@@ -1,6 +1,6 @@
 """
 Provide basic components for groupby. These definitions
-hold the whitelist of methods that are exposed on the
+hold the allowlist of methods that are exposed on the
 SeriesGroupBy and the DataFrameGroupBy objects.
 """
 import collections
@@ -53,7 +53,7 @@ class GroupByMixin:
 # forwarding methods from NDFrames
 plotting_methods = frozenset(["plot", "hist"])
 
-common_apply_whitelist = (
+common_apply_allowlist = (
     frozenset(
         [
             "quantile",
@@ -72,9 +72,9 @@ common_apply_whitelist = (
     | plotting_methods
 )
 
-series_apply_whitelist = (
+series_apply_allowlist = (
     (
-        common_apply_whitelist
+        common_apply_allowlist
         | {
             "nlargest",
             "nsmallest",
@@ -84,13 +84,13 @@ series_apply_whitelist = (
     )
 ) | frozenset(["dtype", "unique"])
 
-dataframe_apply_whitelist = common_apply_whitelist | frozenset(["dtypes", "corrwith"])
+dataframe_apply_allowlist = common_apply_allowlist | frozenset(["dtypes", "corrwith"])
 
 # cythonized transformations or canned "agg+broadcast", which do not
 # require postprocessing of the result by transform.
 cythonized_kernels = frozenset(["cumprod", "cumsum", "shift", "cummin", "cummax"])
 
-cython_cast_blacklist = frozenset(["rank", "count", "size", "idxmin", "idxmax"])
+cython_cast_blocklist = frozenset(["rank", "count", "size", "idxmin", "idxmax"])
 
 # List of aggregation/reduction functions.
 # These map each group to a single numeric value
@@ -98,6 +98,7 @@ reduction_kernels = frozenset(
     [
         "all",
         "any",
+        "corrwith",
         "count",
         "first",
         "idxmax",
@@ -132,7 +133,6 @@ transformation_kernels = frozenset(
     [
         "backfill",
         "bfill",
-        "corrwith",
         "cumcount",
         "cummax",
         "cummin",
@@ -180,9 +180,10 @@ groupby_other_methods = frozenset(
         "tail",
         "take",
         "transform",
+        "sample",
     ]
 )
 # Valid values  of `name` for `groupby.transform(name)`
 # NOTE: do NOT edit this directly. New additions should be inserted
 # into the appropriate list above.
-transform_kernel_whitelist = reduction_kernels | transformation_kernels
+transform_kernel_allowlist = reduction_kernels | transformation_kernels
