@@ -3,6 +3,7 @@ import pytest
 
 from pandas._libs import lib, writers as libwriters
 
+import pandas as pd
 from pandas import Index
 import pandas._testing as tm
 
@@ -22,7 +23,8 @@ class TestMisc:
         assert libwriters.max_len_string_array(arr) == 3
 
         # raises
-        with pytest.raises(TypeError):
+        msg = "No matching signature found"
+        with pytest.raises(TypeError, match=msg):
             libwriters.max_len_string_array(arr.astype("U"))
 
     def test_fast_unique_multiple_list_gen_sort(self):
@@ -38,13 +40,18 @@ class TestMisc:
         out = lib.fast_unique_multiple_list_gen(gen, sort=False)
         tm.assert_numpy_array_equal(np.array(out), expected)
 
+    def test_fast_unique_multiple_unsortable_runtimewarning(self):
+        arr = [np.array(["foo", pd.Timestamp("2000")])]
+        with tm.assert_produces_warning(RuntimeWarning):
+            lib.fast_unique_multiple(arr, sort=None)
+
 
 class TestIndexing:
     def test_maybe_indices_to_slice_left_edge(self):
         target = np.arange(100)
 
         # slice
-        indices = np.array([], dtype=np.int64)
+        indices = np.array([], dtype=np.intp)
         maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
         assert isinstance(maybe_slice, slice)
@@ -52,7 +59,7 @@ class TestIndexing:
 
         for end in [1, 2, 5, 20, 99]:
             for step in [1, 2, 4]:
-                indices = np.arange(0, end, step, dtype=np.int64)
+                indices = np.arange(0, end, step, dtype=np.intp)
                 maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
                 assert isinstance(maybe_slice, slice)
@@ -67,7 +74,7 @@ class TestIndexing:
 
         # not slice
         for case in [[2, 1, 2, 0], [2, 2, 1, 0], [0, 1, 2, 1], [-2, 0, 2], [2, 0, -2]]:
-            indices = np.array(case, dtype=np.int64)
+            indices = np.array(case, dtype=np.intp)
             maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
             assert not isinstance(maybe_slice, slice)
@@ -80,7 +87,7 @@ class TestIndexing:
         # slice
         for start in [0, 2, 5, 20, 97, 98]:
             for step in [1, 2, 4]:
-                indices = np.arange(start, 99, step, dtype=np.int64)
+                indices = np.arange(start, 99, step, dtype=np.intp)
                 maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
                 assert isinstance(maybe_slice, slice)
@@ -94,30 +101,32 @@ class TestIndexing:
                 tm.assert_numpy_array_equal(target[indices], target[maybe_slice])
 
         # not slice
-        indices = np.array([97, 98, 99, 100], dtype=np.int64)
+        indices = np.array([97, 98, 99, 100], dtype=np.intp)
         maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
         assert not isinstance(maybe_slice, slice)
         tm.assert_numpy_array_equal(maybe_slice, indices)
 
-        with pytest.raises(IndexError):
+        msg = "index 100 is out of bounds for axis (0|1) with size 100"
+
+        with pytest.raises(IndexError, match=msg):
             target[indices]
-        with pytest.raises(IndexError):
+        with pytest.raises(IndexError, match=msg):
             target[maybe_slice]
 
-        indices = np.array([100, 99, 98, 97], dtype=np.int64)
+        indices = np.array([100, 99, 98, 97], dtype=np.intp)
         maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
         assert not isinstance(maybe_slice, slice)
         tm.assert_numpy_array_equal(maybe_slice, indices)
 
-        with pytest.raises(IndexError):
+        with pytest.raises(IndexError, match=msg):
             target[indices]
-        with pytest.raises(IndexError):
+        with pytest.raises(IndexError, match=msg):
             target[maybe_slice]
 
         for case in [[99, 97, 99, 96], [99, 99, 98, 97], [98, 98, 97, 96]]:
-            indices = np.array(case, dtype=np.int64)
+            indices = np.array(case, dtype=np.intp)
             maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
             assert not isinstance(maybe_slice, slice)
@@ -129,7 +138,7 @@ class TestIndexing:
 
         # slice
         for step in [1, 2, 4, 5, 8, 9]:
-            indices = np.arange(0, 9, step, dtype=np.int64)
+            indices = np.arange(0, 9, step, dtype=np.intp)
             maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
             assert isinstance(maybe_slice, slice)
             tm.assert_numpy_array_equal(target[indices], target[maybe_slice])
@@ -142,7 +151,7 @@ class TestIndexing:
 
         # not slice
         for case in [[4, 2, 0, -2], [2, 2, 1, 0], [0, 1, 2, 1]]:
-            indices = np.array(case, dtype=np.int64)
+            indices = np.array(case, dtype=np.intp)
             maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
             assert not isinstance(maybe_slice, slice)
             tm.assert_numpy_array_equal(maybe_slice, indices)
@@ -154,7 +163,7 @@ class TestIndexing:
         # slice
         for start, end in [(2, 10), (5, 25), (65, 97)]:
             for step in [1, 2, 4, 20]:
-                indices = np.arange(start, end, step, dtype=np.int64)
+                indices = np.arange(start, end, step, dtype=np.intp)
                 maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
                 assert isinstance(maybe_slice, slice)
@@ -169,7 +178,7 @@ class TestIndexing:
 
         # not slice
         for case in [[14, 12, 10, 12], [12, 12, 11, 10], [10, 11, 12, 11]]:
-            indices = np.array(case, dtype=np.int64)
+            indices = np.array(case, dtype=np.intp)
             maybe_slice = lib.maybe_indices_to_slice(indices, len(target))
 
             assert not isinstance(maybe_slice, slice)
